unit ConsumerToolUnit;

interface

uses
  BTJMSInterfaces;

type
{$M+}
  TConsumerTool = class(TInterfacedObject, IMessageListener)
  private
    Session: ISession;
    Running: Boolean;
    Consumer: IMessageConsumer;
    ReplyProducer: IMessageProducer;

    FAckMode: TAcknowledgementMode;
    FURL: string;
    FTopic: Boolean;
    FSubject: string;
    FDurable: Boolean;
    FSleepTime: Integer;
    FMaximumMessages: Integer;
    FTransacted: Boolean;
    FVerbose: Boolean;
    FUser: string;
    FPassword: string;
    FClientId: string;
    FConsumerName: string;
    FReceiveTimeOut: Integer;
    FPauseBeforeShutdown: Boolean;

    function TargetType: string;
    function DurableString: string;

    procedure SetAckMode(const Value: string);

    procedure OnMessage(const Message: IMessage);
    procedure ConsumeMessagesAndClose(Conn: IConnection; Session: ISession;
      Consumer: IMessageConsumer); overload;
    procedure ConsumeMessagesAndClose(Conn: IConnection; Session: ISession;
      Consumer: IMessageConsumer; TimeOut: Integer); overload;

  public
    constructor Create;

    procedure Run;

  published
    property AckMode: string write SetAckMode;
    property ClientId: string read FClientId write FClientId;
    property ConsumerName: string read FConsumerName write FConsumerName;
    property Durable: Boolean read FDurable write FDurable;
    property MaximumMessages: Integer read FMaximumMessages write
      FMaximumMessages;
    property Password: string read FPassword write FPassword;
    property PauseBeforeShutdown: Boolean read FPauseBeforeShutdown write
      FPauseBeforeShutdown;
    property ReceiveTimeOut: Integer read FReceiveTimeOut write FReceiveTimeOut;
    property SleepTime: Integer read FSleepTime write FSleepTime;
    property Subject: string read FSubject write FSubject;
    property Topic: Boolean read FTopic write FTopic;
    property Transacted: Boolean read FTransacted write FTransacted;
    property URL: string read FURL write FURL;
    property User: string read FUser write FUser;
    property Verbose: Boolean read FVerbose write FVerbose;

  end;

implementation

uses
  CommandLineSupport,
  BTCommAdapterIndy,
  BTJMSConnection,
  BTJMSConnectionFactory,
  StrUtils, SysUtils;

{ TConsumerTool }

constructor TConsumerTool.Create;
begin
  ConsumerName := 'James';
  MaximumMessages := 10;
  Subject := 'TOOL.DEFAULT';
  URL := BTJMSConnectionFactory.DEFAULT_BROKER_URL;
  Verbose := True;
end;

procedure TConsumerTool.SetAckMode(const Value: string);
begin
  if Value = 'CLIENT_ACKNOWLEDGE' then
    FAckMode := amClientAcknowledge
  else if Value = 'AUTO_ACKNOWLEDGE' then
    FAckMode := amAutoAcknowledge
  else if Value = 'SESSION_TRANSACTED' then
    FAckMode := amTransactional
end;

function TConsumerTool.TargetType: string;
begin
  if Topic then
    Result := 'topic'
  else
    Result := 'queue';
end;

function TConsumerTool.DurableString: string;
begin
  if Durable then
    Result := 'durable'
  else
    Result := 'non-durable';
end;

procedure TConsumerTool.OnMessage(const Message: IMessage);
var
  TxtMsg: ITextMessage;
  Msg: string;
begin
  try
    try
      if Supports(Message, ITextMessage, TxtMsg) then
      begin
        if Verbose then
        begin
          Msg := TxtMsg.Text;
          if Length(Msg) > 50 then
            Msg := Copy(Msg, 1, 50) + '...';
          WriteLn('Received: ' + Msg);
        end;
      end
      else
      begin
        if Verbose then
          WriteLn('Received: Message');
      end;

      if Message.JMSReplyTo <> nil then
      begin
        ReplyProducer.Send(Message.JMSReplyTo,
          Session.CreateTextMessage('Reply: ' + Message.JMSMessageID));
      end;

      if Transacted then
        Session.Commit
      else if FAckMode = amClientAcknowledge then
        Message.Acknowledge;

    except
      on E: Exception do
      begin
        WriteLn(E.Message);
      end;

    end;
  finally
    if SleepTime > 0 then
    begin
      Sleep(SleepTime);
    end;
  end;

end;

procedure TConsumerTool.ConsumeMessagesAndClose(Conn: IConnection; Session:
  ISession; Consumer: IMessageConsumer);
var
  I: Integer;
  Message: IMessage;
begin
  WriteLn('We are about to wait until we consume: ' + IntToStr(MaximumMessages)
    + ' message(s) then we will shutdown');

  I := 0;
  while (I < MaximumMessages) and Running do
  begin
    Message := Consumer.Receive(1000);
    if Message <> nil then
    begin
      Inc(I);
      OnMessage(Message);
    end;
  end;

  WriteLn('Closing connection');
  Consumer.Close;
  Session.Close;
  Conn.Close;
  if PauseBeforeShutdown then
  begin
    WriteLn('Press return to shut down');
    ReadLn;
  end;
end;

procedure TConsumerTool.ConsumeMessagesAndClose(Conn: IConnection; Session:
  ISession; Consumer: IMessageConsumer; TimeOut: Integer);
var
  Message: IMessage;
begin
  WriteLn('We will consume messages while they continue to be delivered within: '
    + IntToStr(Timeout) + ' ms, and then we will shutdown');

  Message := Consumer.Receive(Timeout);
  while (Message <> nil) do
  begin
    OnMessage(Message);
    Message := Consumer.Receive(Timeout);
  end;

  WriteLn('Closing connection');
  Consumer.Close;
  Session.Close;
  Conn.Close;
  if PauseBeforeShutdown then
  begin
    WriteLn('Press return to shut down');
    ReadLn;
  end;

end;

procedure TConsumerTool.Run;
var
  ConnectionFactory: TBTJMSConnectionFactory;
  Connection: IConnection;

  Destination: IDestination;
begin
  TCommandLineSupport.Configure(Self);

  Running := True;

  WriteLn('Connecting to URL: ' + URL);
  WriteLn('Consuming ' + TargetType + ': ' + Subject);
  WriteLn('Using a ' + DurableString + ' subscription');

  ConnectionFactory := TBTJMSConnectionFactory.Create(User, Password, URL);
  Connection := ConnectionFactory.CreateConnection;
  if (Durable and (ClientId <> '')) then
  begin
    Connection.ClientID := ClientId;
  end;
  Connection.Start;

  // Create the session.
  Session := Connection.CreateSession(Transacted, FAckMode);

  // Create the Producer for the Destination.
  if Topic then
    Destination := Session.CreateTopic(Subject)
  else
    Destination := Session.CreateQueue(Subject);

  ReplyProducer := Session.createProducer(nil);
  ReplyProducer.setDeliveryMode(dmNonPersistent);

  if (Durable and Topic) then
    Consumer := Session.CreateDurableSubscriber(ITopic(Destination),
      ConsumerName)
  else
    Consumer := Session.CreateConsumer(Destination);

  if MaximumMessages > 0 then
  begin
    ConsumeMessagesAndClose(Connection, Session, Consumer);
  end
  else
  begin
    if ReceiveTimeOut = 0 then
      Consumer.SetMessageListener(Self)
    else
      ConsumeMessagesAndClose(Connection, Session, Consumer, ReceiveTimeOut);
  end;

  Connection.Close;
  WriteLn('Done.');
end;

end.

