unit ProducerToolUnit;

interface

uses
  BTJMSInterfaces;

type
{$M+}
  TProducerTool = class(TObject)
  private
    FURL: string;
    FMessageSize: Integer;
    FTopic: Boolean;
    FSubject: string;
    FPersistent: Boolean;
    FSleepTime: Integer;
    FTimeToLive: Integer;
    FMessageCount: Integer;
    FTransacted: Boolean;
    FVerbose: Boolean;

    function TargetType: string;
    function PersistentString: string;

    procedure SendLoop(const Session: ISession;
      const Producer: IMessageProducer);

  public
    constructor Create;

    procedure Run;

  published
    property MessageCount: Integer read FMessageCount write FMessageCount;
    property MessageSize: Integer read FMessageSize write FMessageSize;
    property Persistent: Boolean read FPersistent write FPersistent;
    property SleepTime: Integer read FSleepTime write FSleepTime;
    property Subject: string read FSubject write FSubject;
    property TimeToLive: Integer read FTimeToLive write FTimeToLive;
    property Topic: Boolean read FTopic write FTopic;
    property Transacted: Boolean read FTransacted write FTransacted;
    property URL: string read FURL write FURL;
    property Verbose: Boolean read FVerbose write FVerbose;

  end;

implementation

uses
  CommandLineSupport,
  BTCommAdapterIndy,
  BTJMSConnection,
  BTJMSConnectionFactory,
  StrUtils, SysUtils;

{ TProducerTool }

constructor TProducerTool.Create;
begin
  MessageCount := 10;
  MessageSize := 255;
  Subject := 'TOOL.DEFAULT';
  URL := BTJMSConnectionFactory.DEFAULT_BROKER_URL;
  Verbose := True;
end;

function TProducerTool.TargetType: string;
begin
  if Topic then
    Result := 'topic'
  else
    Result := 'queue';
end;

function TProducerTool.PersistentString: string;
begin
  if Persistent then
    Result := 'persistent'
  else
    Result := 'non-persistent';
end;

procedure TProducerTool.Run;
var
  Connection: IConnection;
  Session: ISession;
  Destination: IDestination;
  Producer: IMessageProducer;
begin
  TCommandLineSupport.Configure(Self);

  WriteLn('Connecting to URL: ' + URL);
  WriteLn('Publishing a Message with size ' + IntToStr(MessageSize) + ' to ' +
    TargetType + ': ' + Subject);
  WriteLn('Using ' + PersistentString + ' messages');
  WriteLn('Sleeping between publish ' + IntToStr(SleepTime) + ' ms');
  if TimeToLive <> 0 then
  begin
    WriteLn('Messages time to live ' + IntToStr(TimeToLive) + ' ms');
  end;

  Connection := TBTJMSConnection.MakeConnection;
  Connection.Start;

  // Create the session.
  Session := Connection.CreateSession(Transacted, amAutoAcknowledge);

  // Create the Producer for the Destination.
  if Topic then
    Destination := Session.CreateTopic(Subject)
  else
    Destination := Session.CreateQueue(Subject);

  // Create the producer.
  Producer := Session.CreateProducer(Destination);

  if Persistent then
    Producer.DeliveryMode := dmPersistent
  else
    Producer.DeliveryMode := dmNonPersistent;

  if TimeToLive <> 0 then
    Producer.TimeToLive := TimeToLive;

  SendLoop(Session, Producer);

  Connection.Close;
  WriteLn('Done.');
end;

procedure TProducerTool.SendLoop(const Session: ISession;
  const Producer: IMessageProducer);
var
  I: Integer;
  TextMessage: ITextMessage;
  Msg: string;

  function CreateMessageText(const Index: Integer): string;
  begin
    Result := 'Message: ' + IntToStr(Index) + ' sent at: ' + DateTimeToStr(Now);

    if Length(Result) > MessageSize then
      Result := Copy(Result, 1, MessageSize)
    else
      Result := Copy(Result + DupeString(' ', MessageSize), 1, MessageSize);
  end;

begin
  for I := 0 to MessageCount - 1 do
  begin
    TextMessage := Session.CreateTextMessage(CreateMessageText(I));
    if Verbose then
    begin
      Msg := TextMessage.Text;
      if Length(Msg) > 50 then
      begin
        Msg := Copy(Msg, 1, 50) + '...';
      end;
      WriteLn('Sending message: ' + Msg);
    end;
    Producer.Send(TextMessage);
    if Transacted then
    begin
      Session.Commit;
    end;
    Sleep(SleepTime);
  end;
end;

end.

